==> node-red-contrib-waveshare-shield-adda-11010-1.2.0.tgz: package/ads1256.py <==
#!/usr/bin/env python3
# ads1256.py
#
# Copyright (c) 2020-2022 Spring City Solutions LLC and other contributors
# Licensed under the Apache License, Version 2.0
#
# Need to have the rpi.gpio and spidev python libraries installed
#
# Usage:
#
# ads1256.py analog "channel number 0 to 7" "gain 1 to 64" "rate 2.5 to 30000"
# returns 0 for 0 volts and decimal 0x7fffff for 5 volts
#
# ads1256.py calibrationFSC
# returns fullscale calibration register value as a decimal integer 
#
# ads1256.py calibrationOFC
# returns offset calibration register value as a decimal integer 
#
# ads1256.py digitalIn "D0 to D3"
# returns 0 or 1 as per input to the specified pin
#
# ads1256.py digitalOut "D0 to D3" "0 or 1"
#
# Hardware Links:
#
# Board Manufacturer Store:
# https://www.waveshare.com/High-Precision-AD-DA-Board.htm
#
# Board Manufacturer Wiki:
# https://www.waveshare.com/wiki/High-Precision_AD/DA_Board
#
# Links for ADS1256 chip:
# https://www.ti.com/product/ADS1256
# https://www.ti.com/lit/gpn/ads1256
# Book "Fundamentals of Precision ADC Noise Analysis" https://www.ti.com/lit/pdf/slyy192
# Journal Series "How delta-sigma ADCs work" https://www.ti.com/lit/pdf/slyt423 https://www.ti.com/lit/pdf/slyt438
# App Note "Digital Filter Types in Delta-Sigma ADCs" https://www.ti.com/lit/pdf/sbaa230
#
# Link for LM285 chip, connected to the ADS1256:
# https://www.ti.com/product/LM285-2.5-N
# https://www.ti.com/lit/gpn/lm285-2.5-n
#
# Link for DAC8532 chip:
# https://www.ti.com/product/DAC8532
# https://www.ti.com/lit/gpn/dac8532
#
# Spring City Solutions Links:
#
# Spring City Solutions Node-RED project page:
# https://www.springcitysolutions.com/nodered
#
# Spring City Solutions page for this node:
# https://www.springcitysolutions.com/nodered-waveshare-adda-shield
#
# Spring City Solutions Node-RED project email:
# nodered@springcitysolutions.com
# 
# Spring City Solutions Gitlab for this node:
# https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
#
# Patreon Page:
# https://www.patreon.com/springcitysolutions_nodered
#
# Software Links:
#
# Spring City Solutions Gitlab for this node:
# https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
#
# npmjs for this node:
# https://npmjs.org/package/node-red-contrib-waveshare-shield-adda-11010
#
# Node-RED flows for this node:
# https://flows.nodered.org/node/node-red-contrib-waveshare-shield-adda-11010
#
# Documentation Links:
#
# Gitlab wiki for this node (the master copy of list of links is here):
# https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010/-/wikis/home
#
# Waveshare's libraries for this hardware:
# https://github.com/waveshare/High-Precision-AD-DA-Board
#
# Youtube video "How to set up":
# TODO
#
# Youtube video "How to use":
# TODO
#
# Youtube video "Testing Results":
# TODO

import RPi.GPIO as GPIO
import spidev
import sys

cs = 22
drdy = 17
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
GPIO.setup(cs, GPIO.OUT)
GPIO.output(cs, 1)
GPIO.setup(drdy, GPIO.IN)

spi = spidev.SpiDev()
spi.open(0, 0)
spi.max_speed_hz = 20000
spi.mode = 0b01


def ReadReg(register):
    GPIO.output(cs, 0)
    firstbyte = 0x10 + register
    secondbyte = 0x01
    spi.writebytes([firstbyte, secondbyte])
    data = spi.readbytes(1)
    GPIO.output(cs, 1)
    return data[0]


def PrintRegisters():
    print('0x00 0x' + format(ReadReg(0x00), '02x') + ' Status')
    print('0x01 0x' + format(ReadReg(0x01), '02x') + ' MUX')
    print('0x02 0x' + format(ReadReg(0x02), '02x') + ' ADCON')
    print('0x03 0x' + format(ReadReg(0x03), '02x') + ' DRATE')
    print('0x04 0x' + format(ReadReg(0x04), '02x') + ' IO')
    print('0x05 0x' + format(ReadReg(0x05), '02x') + ' OFC0')
    print('0x06 0x' + format(ReadReg(0x06), '02x') + ' OFC1')
    print('0x07 0x' + format(ReadReg(0x07), '02x') + ' OFC2')
    print('0x08 0x' + format(ReadReg(0x08), '02x') + ' FSC0')
    print('0x09 0x' + format(ReadReg(0x09), '02x') + ' FSC1')
    print('0x0A 0x' + format(ReadReg(0x0A), '02x') + ' FSC2')
    print()


def WriteReg(register, data):
    GPIO.output(cs, 0)
    firstbyte = 0x50 + register
    secondbyte = 0x01
    spi.writebytes([firstbyte, secondbyte, data])
    GPIO.output(cs, 1)


def WriteCmd(command):
    GPIO.output(cs, 0)
    spi.writebytes([command])
    GPIO.output(cs, 1)


def SELFCAL():
    WriteCmd(0xf0)
    while True:
        if (GPIO.input(drdy) == 0):
            break


def RDATA():
    while True:
        if (GPIO.input(drdy) == 0):
            break
    GPIO.output(cs, 0)
    spi.writebytes([0x01])
    buf = spi.readbytes(3)
    GPIO.output(cs, 1)
    read = (buf[0] << 16) & 0xff0000
    read |= (buf[1] << 8) & 0xff00
    read |= (buf[2]) & 0xff
    if (read & 0x800000):
        read &= 0xF000000
    return read


# Shut off pin D0 clock out, shut off SDCS, set gain to x1
WriteReg(0x02, 0x00)

if (sys.argv[1] == 'analog'):
    # Set the ADCON register
    adcon = 0x20
    if (sys.argv[3] == '1'):
        adcon=0x20
    if (sys.argv[3] == '2'):
        adcon=0x21    
    if (sys.argv[3] == '4'):
        adcon=0x22
    if (sys.argv[3] == '8'):
        adcon=0x23
    if (sys.argv[3] == '16'):
        adcon=0x24
    if (sys.argv[3] == '32'):
        adcon=0x25
    if (sys.argv[3] == '64'):
        adcon=0x26
    WriteReg(0x02, adcon)
    # Set the DRATE register
    drate = 0x03
    if (sys.argv[4] == '30000'):
        drate = 0xF0
    if (sys.argv[4] == '15000'):
        drate = 0xE0
    if (sys.argv[4] == '7500'):
        drate = 0xD0
    if (sys.argv[4] == '3750'):
        drate = 0xC0
    if (sys.argv[4] == '2000'):
        drate = 0xB0
    if (sys.argv[4] == '1000'):
        drate = 0xA1
    if (sys.argv[4] == '500'):
        drate = 0x91
    if (sys.argv[4] == '100'):
        drate = 0x82
    if (sys.argv[4] == '60'):
        drate = 0x72
    if (sys.argv[4] == '50'):
        drate = 0x63
    if (sys.argv[4] == '30'):
        drate = 0x53
    if (sys.argv[4] == '25'):
        drate = 0x43
    if (sys.argv[4] == '15'):
        drate = 0x33
    if (sys.argv[4] == '10'):
        drate = 0x23
    if (sys.argv[4] == '5'):
        drate = 0x13        
    if (sys.argv[4] == '2.5'):
        drate = 0x03
    WriteReg(0x03, drate)
    # Set the MUX register
    channelbyte = ((int(sys.argv[2])) << 4) | (1 << 3)
    WriteReg(0x01, channelbyte)
    # Self Calibration required
    SELFCAL()
    # PrintRegisters()
    # Sync and wakeup the converter
    WriteCmd(0xfc)
    WriteCmd(0xff)
    # Read data
    print(RDATA())

if (sys.argv[1] == 'calibrationFSC'):
    # FSC per datasheet pg 24 is an unsigned integer
    fsc_hex = format(ReadReg(0x0A), '02x') + format(ReadReg(0x09), '02x') + format(ReadReg(0x08), '02x')
    fsc_dec = int(fsc_hex, 24)
    print(fsc_dec)

if (sys.argv[1] == 'calibrationOFC'):
    # OFC per datasheet pg 24 is a twos complement signed integer
    ofc_hex = format(ReadReg(0x07), '02x') + format(ReadReg(0x06), '02x') + format(ReadReg(0x05), '02x')
    ofc_dec = -(int(ofc_hex, 24) & 0x800000) | (int(ofc_hex, 24) & 0x7fffff)
    print(ofc_dec)

if (sys.argv[1] == 'digitalIn'):
    directionbitmask = 0x10
    inputbitmask = 0x01
    if (sys.argv[2] == 'D0'):
        directionbitmask = 0x10
        inputbitmask = 0x01
    if (sys.argv[2] == 'D1'):
        directionbitmask = 0x20
        inputbitmask = 0x02
    if (sys.argv[2] == 'D2'):
        directionbitmask = 0x40
        inputbitmask = 0x04
    if (sys.argv[2] == 'D3'):
        directionbitmask = 0x80
        inputbitmask = 0x08
    oldregister = ReadReg(0x04)
    WriteReg(0x04, oldregister | directionbitmask)
    if ((ReadReg(0x04) & inputbitmask) > 0):
        print('1')
    else:
        print('0')

if (sys.argv[1] == 'digitalOut'):
    directionbitmask = 0x10
    outputbitmask = 0x01
    if (sys.argv[2] == 'D0'):
        directionbitmask = 0x10
        outputbitmask = 0x01
    if (sys.argv[2] == 'D1'):
        directionbitmask = 0x20
        outputbitmask = 0x02
    if (sys.argv[2] == 'D2'):
        directionbitmask = 0x40
        outputbitmask = 0x04
    if (sys.argv[2] == 'D3'):
        directionbitmask = 0x80
        outputbitmask = 0x08
    oldregister = ReadReg(0x04)
    if (sys.argv[3] == '0'):
        WriteReg(0x04, oldregister & ~directionbitmask & ~outputbitmask)
    if (sys.argv[3] == '1'):
        WriteReg(0x04, oldregister & ~directionbitmask | outputbitmask)

==> node-red-contrib-waveshare-shield-adda-11010-1.5.0.tgz: package/ads1256.py <==
#!/usr/bin/env python3
# ads1256.py
#
# Copyright (c) 2020-2022 Spring City Solutions LLC and other contributors
# Licensed under the Apache License, Version 2.0
#
# Need to have the rpi.gpio and spidev python libraries installed
#
# Usage:
#
# ads1256.py analog "channel number 0 to 7" "gain 1 to 64" "rate 2.5 to 30000" "on or off buffer" "sdcs off of current"
# returns 0 for 0 volts and decimal 0x7fffff for 5 volts
#
# ads1256.py calibrationFSC
# returns fullscale calibration register value as a decimal integer 
#
# ads1256.py calibrationOFC
# returns offset calibration register value as a decimal integer 
#
# ads1256.py digitalIn "D0 to D3"
# returns 0 or 1 as per input to the specified pin
#
# ads1256.py digitalOut "D0 to D3" "0 or 1"
#
# Hardware Links:
#
# Board Manufacturer Store:
# https://www.waveshare.com/High-Precision-AD-DA-Board.htm
#
# Board Manufacturer Wiki:
# https://www.waveshare.com/wiki/High-Precision_AD/DA_Board
#
# Links for ADS1256 chip:
# https://www.ti.com/product/ADS1256
# https://www.ti.com/lit/gpn/ads1256
# Book "Fundamentals of Precision ADC Noise Analysis" https://www.ti.com/lit/pdf/slyy192
# Journal Series "How delta-sigma ADCs work" https://www.ti.com/lit/pdf/slyt423 https://www.ti.com/lit/pdf/slyt438
# App Note "Digital Filter Types in Delta-Sigma ADCs" https://www.ti.com/lit/pdf/sbaa230
#
# Link for LM285 chip, connected to the ADS1256:
# https://www.ti.com/product/LM285-2.5-N
# https://www.ti.com/lit/gpn/lm285-2.5-n
#
# Link for DAC8532 chip:
# https://www.ti.com/product/DAC8532
# https://www.ti.com/lit/gpn/dac8532
#
# Spring City Solutions Links:
#
# Spring City Solutions Node-RED project page:
# https://www.springcitysolutions.com/nodered
#
# Spring City Solutions page for this node:
# https://www.springcitysolutions.com/nodered-waveshare-adda-shield
#
# Spring City Solutions Node-RED project email:
# nodered@springcitysolutions.com
# 
# Spring City Solutions Gitlab for this node:
# https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
#
# Patreon Page:
# https://www.patreon.com/springcitysolutions_nodered
#
# Software Links:
#
# Spring City Solutions Gitlab for this node:
# https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
#
# npmjs for this node:
# https://npmjs.org/package/node-red-contrib-waveshare-shield-adda-11010
#
# Node-RED flows for this node:
# https://flows.nodered.org/node/node-red-contrib-waveshare-shield-adda-11010
#
# Documentation Links:
#
# Gitlab wiki for this node (the master copy of list of links is here):
# https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010/-/wikis/home
#
# Waveshare's libraries for this hardware:
# https://github.com/waveshare/High-Precision-AD-DA-Board
#
# Youtube video "How to set up":
# TODO
#
# Youtube video "How to use":
# TODO
#
# Youtube video "Testing Results":
# TODO

import sys

import RPi.GPIO as GPIO
import spidev

cs = 22
drdy = 17

# TODO: Eliminate the register addresses as magic numbers by placing a table of named constants here.

GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
GPIO.setup(cs, GPIO.OUT)
GPIO.output(cs, 1)
GPIO.setup(drdy, GPIO.IN)

spi = spidev.SpiDev()
spi.open(0, 0)
spi.max_speed_hz = 20000
spi.mode = 0b01

# Device commands are run by functions named after the device commands.
# The datasheet names are all caps, Python names are lowercase.
# The complete list of commands is on datasheet page 34 table 24.


def rreg(register):
    """Run RREG command as explained on datasheet page 36."""
    GPIO.output(cs, 0)
    firstbyte = 0x10 + register
    secondbyte = 0x01
    spi.writebytes([firstbyte, secondbyte])
    data = spi.readbytes(1)
    GPIO.output(cs, 1)
    return data[0]


def wreg(register, data):
    """Run WREG command as explained on datasheet page 36."""
    GPIO.output(cs, 0)
    firstbyte = 0x50 + register
    secondbyte = 0x01
    spi.writebytes([firstbyte, secondbyte, data])
    GPIO.output(cs, 1)


def selfcal():
    """Run SELFCAL command as explained on datasheet page 36."""
    GPIO.output(cs, 0)
    spi.writebytes([0xf0])
    GPIO.output(cs, 1)
    while True:
        if (GPIO.input(drdy) == 0):
            break


def rdata():
    """Run RDATA command as explained on datasheet page 34."""
    while True:
        if (GPIO.input(drdy) == 0):
            break
    GPIO.output(cs, 0)
    spi.writebytes([0x01])
    buf = spi.readbytes(3)
    GPIO.output(cs, 1)
    read = (buf[0] << 16) & 0xff0000
    read |= (buf[1] << 8) & 0xff00
    read |= (buf[2]) & 0xff
    if (read & 0x800000):
        read &= 0xF000000
    return read


def sync():
    """Run SYNC command as explained on datasheet page 37."""
    GPIO.output(cs, 0)
    spi.writebytes([0xfc])
    GPIO.output(cs, 1)


def wakeup():
    """Run WAKEUP command as explained on datasheet page 37."""
    GPIO.output(cs, 0)
    spi.writebytes([0xff])
    GPIO.output(cs, 1)


# TODO: Rework how args are handled.

# TODO: Output a how to use message if no args are provided.

if (sys.argv[1] == 'analog'):
    # Set the STATUS register, BUFEN analog input buffer property.
    status = 0x00
    if (sys.argv[5] == 'on'):
        status=0x02
    if (sys.argv[5] == 'off'):
        status=0x00       
    wreg(0x00, status)
    # Set the ADCON register, PGA gain and SDCS property.
    adcon = 0x20 # Default clock out rate setting
    if (sys.argv[3] == '1'):
        pga=0x00
    if (sys.argv[3] == '2'):
        pga=0x01    
    if (sys.argv[3] == '4'):
        pga=0x02
    if (sys.argv[3] == '8'):
        pga=0x03
    if (sys.argv[3] == '16'):
        pga=0x04
    if (sys.argv[3] == '32'):
        pga=0x05
    if (sys.argv[3] == '64'):
        pga=0x06
    if (sys.argv[6] == 'off'):
        sdcs=0x00
    if (sys.argv[6] == '0.5'):
        sdcs=0x08
    if (sys.argv[6] == '2'):
        sdcs=0x10
    if (sys.argv[6] == '10'):
        sdcs=0x18
    adcon = adcon + pga + sdcs
    wreg(0x02, adcon)
    # Set the DRATE register, samples per second property.
    drate = 0x03
    if (sys.argv[4] == '30000'):
        drate = 0xF0
    if (sys.argv[4] == '15000'):
        drate = 0xE0
    if (sys.argv[4] == '7500'):
        drate = 0xD0
    if (sys.argv[4] == '3750'):
        drate = 0xC0
    if (sys.argv[4] == '2000'):
        drate = 0xB0
    if (sys.argv[4] == '1000'):
        drate = 0xA1
    if (sys.argv[4] == '500'):
        drate = 0x91
    if (sys.argv[4] == '100'):
        drate = 0x82
    if (sys.argv[4] == '60'):
        drate = 0x72
    if (sys.argv[4] == '50'):
        drate = 0x63
    if (sys.argv[4] == '30'):
        drate = 0x53
    if (sys.argv[4] == '25'):
        drate = 0x43
    if (sys.argv[4] == '15'):
        drate = 0x33
    if (sys.argv[4] == '10'):
        drate = 0x23
    if (sys.argv[4] == '5'):
        drate = 0x13        
    if (sys.argv[4] == '2.5'):
        drate = 0x03
    wreg(0x03, drate)
    # Set the MUX register to select input pin.
    channelbyte = ((int(sys.argv[2])) << 4) | (1 << 3)
    wreg(0x01, channelbyte)
    # Self Calibration required after MUX change.
    selfcal()
    # Sync and Wakeup, in that order, are required to start a conversion.
    sync()
    wakeup()
    # Read data.
    print(rdata())

if (sys.argv[1] == 'calibrationFSC'):
    # FSC per datasheet pg 24 is an unsigned integer.
    # TODO: This could be simplified to add/multiply but was developed this way.
    fsc_hex = (format(rreg(0x0A), '02x') 
                + format(rreg(0x09), '02x') 
                + format(rreg(0x08), '02x'))
    fsc_dec = int(fsc_hex, 24)
    print(fsc_dec)

if (sys.argv[1] == 'calibrationOFC'):
    # OFC per datasheet pg 24 is a twos complement signed integer.
    # TODO: This could be simplified to add/multiply but was developed this way.
    ofc_hex = (format(rreg(0x07), '02x')
                + format(rreg(0x06), '02x')
                + format(rreg(0x05), '02x'))
    ofc_dec = (-(int(ofc_hex, 24) & 0x800000)
                | (int(ofc_hex, 24) & 0x7fffff))
    print(ofc_dec)

if (sys.argv[1] == 'digitalIn'):
    directionbitmask = 0x10
    inputbitmask = 0x01
    if (sys.argv[2] == 'D0'):
        directionbitmask = 0x10
        inputbitmask = 0x01
    if (sys.argv[2] == 'D1'):
        directionbitmask = 0x20
        inputbitmask = 0x02
    if (sys.argv[2] == 'D2'):
        directionbitmask = 0x40
        inputbitmask = 0x04
    if (sys.argv[2] == 'D3'):
        directionbitmask = 0x80
        inputbitmask = 0x08
    oldregister = rreg(0x04)
    wreg(0x04, oldregister | directionbitmask)
    if ((rreg(0x04) & inputbitmask) > 0):
        print('1')
    else:
        print('0')

if (sys.argv[1] == 'digitalOut'):
    directionbitmask = 0x10
    outputbitmask = 0x01
    if (sys.argv[2] == 'D0'):
        directionbitmask = 0x10
        outputbitmask = 0x01
    if (sys.argv[2] == 'D1'):
        directionbitmask = 0x20
        outputbitmask = 0x02
    if (sys.argv[2] == 'D2'):
        directionbitmask = 0x40
        outputbitmask = 0x04
    if (sys.argv[2] == 'D3'):
        directionbitmask = 0x80
        outputbitmask = 0x08
    oldregister = rreg(0x04)
    if (sys.argv[3] == '0'):
        wreg(0x04, 
            oldregister & ~directionbitmask & ~outputbitmask)
    if (sys.argv[3] == '1'):
        wreg(0x04, 
            oldregister & ~directionbitmask | outputbitmask)
